"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RepeatOptions = exports.Assertion = exports.LiteralCharacter = exports.Characters = exports.ExpressionBuilder = void 0;
const builderHelpers_1 = require("./builderHelpers");
/**
 * Build a regular expression.
 * @param callback {ExpressionBuilderCallback} - is a callback where the expression is created in.
 * @param options {ExpressionBuilderOptions} - are the options for the expression builder.
 */
const ExpressionBuilder = (callback, options) => {
    let expression = "";
    let start = "";
    let end = "";
    callback({
        Match: (character) => {
            expression += builderHelpers_1.ParseMatch(character);
        },
        Assert: (assertion) => {
            expression += builderHelpers_1.ParseAssertion(assertion);
        },
        PositiveLookahead: (callback) => {
            const temp = expression;
            expression = "";
            callback();
            end += "(?=" + expression + ")";
            expression = temp;
        },
        NegativeLookahead: (callback) => {
            const temp = expression;
            expression = "";
            callback();
            end += "(?!" + expression + ")";
            expression = temp;
        },
        PositiveLookbehind: (callback) => {
            const temp = expression;
            expression = "";
            callback();
            start += "(?<=" + expression + ")";
            expression = temp;
        },
        NegativeLookbehind: (callback) => {
            const temp = expression;
            expression = "";
            callback();
            start += "(?<!" + expression + ")";
            expression = temp;
        },
        OR: (...callbacks) => {
            const temp = expression;
            expression = "";
            let values = [];
            for (let callback of callbacks) {
                callback();
                values.push("(?:" + expression + ")");
                expression = "";
            }
            expression = temp + "(?:" + values.join("|") + ")";
        },
        InSet: (callback) => {
            const temp = expression;
            expression = "";
            callback();
            expression = temp + "[" + expression + "]";
        },
        NotInSet: (callback) => {
            const temp = expression;
            expression = "";
            callback();
            expression = temp + "[^" + expression + "]";
        },
        To: () => {
            expression += "-";
        },
        Group: (callback) => {
            const temp = expression;
            expression = "";
            callback();
            expression = temp + "(" + expression + ")";
        },
        BackRef: (index) => {
            expression += "\\" + index;
        },
        Repeat: (callback, options) => {
            const temp = expression;
            expression = "";
            callback();
            expression = temp + "(?:" + expression + options.repeatChars + ")";
        }
    });
    return "/" + start + expression + end + "/";
};
exports.ExpressionBuilder = ExpressionBuilder;
/**
 * Special characters that can be used in matches.
 */
var Characters;
(function (Characters) {
    Characters[Characters["ANY"] = 0] = "ANY";
    Characters[Characters["DIGIT"] = 1] = "DIGIT";
    Characters[Characters["NON_DIGIT"] = 2] = "NON_DIGIT";
    Characters[Characters["ALPHANUM"] = 3] = "ALPHANUM";
    Characters[Characters["NON_ALPHANUM"] = 4] = "NON_ALPHANUM";
    Characters[Characters["SPACE"] = 5] = "SPACE";
    Characters[Characters["NON_SPACE"] = 6] = "NON_SPACE";
    Characters[Characters["HTAB"] = 7] = "HTAB";
    Characters[Characters["VTAB"] = 8] = "VTAB";
    Characters[Characters["RETURN"] = 9] = "RETURN";
    Characters[Characters["LINEFEED"] = 10] = "LINEFEED";
    Characters[Characters["FORMFEED"] = 11] = "FORMFEED";
    Characters[Characters["BACKSPACE"] = 12] = "BACKSPACE";
    Characters[Characters["NULL"] = 13] = "NULL";
})(Characters = exports.Characters || (exports.Characters = {}));
(function (Characters) {
    /**
     * Match a control character.
     * @param caret {string} - is a one-length character from A-Z indicating the control character.
     */
    Characters.CONTROL = (caret) => {
        if (caret.length !== 1 || !/[A-Z]/.test(caret.toUpperCase()))
            throw new Error("The caret must be a character in A-Z.");
        return exports.LiteralCharacter("\\c" + caret.toUpperCase());
    };
    /**
     * Match a character by its character code
     * @param hex {string} - is a two-length or four-length string of hexadecimal digits indicating the character code.
     */
    Characters.HEX = (hex) => {
        if (![2, 4].includes(hex.length) || !/^[0-9a-fA-F]{2}$/.test(hex))
            throw new Error("The hex must be two or four characters in 0-9 or A-F");
        return exports.LiteralCharacter((hex.length === 2 ? "\\x" : "\\u") + hex.toLowerCase());
    };
})(Characters = exports.Characters || (exports.Characters = {}));
/**
 * Match a character literally, without escaping.
 * @param character {string} - the character being matched.
 */
const LiteralCharacter = (character) => {
    return { character };
};
exports.LiteralCharacter = LiteralCharacter;
/**
 * An assertion.
 */
var Assertion;
(function (Assertion) {
    Assertion[Assertion["START"] = 0] = "START";
    Assertion[Assertion["END"] = 1] = "END";
    Assertion[Assertion["WORD_BOUNDARY"] = 2] = "WORD_BOUNDARY";
    Assertion[Assertion["NOT_WORD_BOUNDARY"] = 3] = "NOT_WORD_BOUNDARY";
})(Assertion = exports.Assertion || (exports.Assertion = {}));
/**
 * Options for a repeat.
 */
var RepeatOptions;
(function (RepeatOptions) {
    /**
     * Repeat zero or more times.
     * @param greedy {boolean} - if set to true, will match as many times as possible, otherwise as least times as possible.
     */
    RepeatOptions.ZeroOrMore = (greedy = true) => {
        return { repeatChars: "*" + (greedy ? "" : "?") };
    };
    /**
     * Repeat one or more times.
     * @param greedy {boolean} - if set to true, will match as many times as possible, otherwise as least times as possible.
     */
    RepeatOptions.OneOrMore = (greedy = true) => {
        return { repeatChars: "+" + (greedy ? "" : "?") };
    };
    /**
     * Repeat zero or one times.
     */
    RepeatOptions.ZeroOrOne = () => {
        return { repeatChars: "?" };
    };
    /**
     * Repeat exactly some amount of times.
     * @param count {number} - the amount of times to repeat.
     */
    RepeatOptions.Exactly = (count) => {
        return { repeatChars: "{" + count + "}" };
    };
    /**
     * Repeat at least some amount of times.
     * @param count {number} - the minimum amount of times to repeat.
     */
    RepeatOptions.AtLeast = (count) => {
        return { repeatChars: "{" + count + ",}" };
    };
    /**
     * Repeat between some amount of times and another amount of times (inclusive).
     * @param count1 {number} - the lower bounds for the amount of times to repeat.
     * @param count2 {number} - the upper bounds for the amount of times to repeat.
     */
    RepeatOptions.Between = (count1, count2) => {
        return { repeatChars: "{" + count1 + "," + count2 + "}" };
    };
})(RepeatOptions = exports.RepeatOptions || (exports.RepeatOptions = {}));
//# sourceMappingURL=builder.js.map