/**
 * Build a regular expression.
 * @param callback {ExpressionBuilderCallback} - is a callback where the expression is created in.
 * @param options {ExpressionBuilderOptions} - are the options for the expression builder.
 */
export declare const ExpressionBuilder: (callback: ExpressionBuilderCallback, options?: ExpressionBuilderOptions) => string;
/**
 * The functions used to build a regular expression.
 */
export interface ExpressionBuilderFunctions {
    /**
     * Match something.
     */
    Match: Match;
    /**
     * Make an assertion.
     */
    Assert: Assert;
    /**
     * Ensure that something appears after the expression.
     */
    PositiveLookahead: EmptyCallback;
    /**
     * Ensure that something does not appear after the expression.
     */
    NegativeLookahead: EmptyCallback;
    /**
     * Ensure that something appears before the expression.
     */
    PositiveLookbehind: EmptyCallback;
    /**
     * Ensure that something does not appear before the expression.
     */
    NegativeLookbehind: EmptyCallback;
    /**
     * Match one thing or another.
     */
    OR: OR;
    /**
     * Match something in a set of matches.
     */
    InSet: EmptyCallback;
    /**
     * Match something not in a set of matches.
     */
    NotInSet: EmptyCallback;
    /**
     * Used in sets; match any character between the match before and the match after this one (inclusive).
     */
    To: Function;
    /**
     * Creates a capturing group.
     */
    Group: EmptyCallback;
    /**
     * Match the value of a previous capturing group, by its one-based ID.
     */
    BackRef: BackRef;
    /**
     * Repeat a match.
     */
    Repeat: Repeat;
}
export declare type ExpressionBuilderCallback = (functions: ExpressionBuilderFunctions) => void;
/**
 * The options for the expression builder.
 */
export interface ExpressionBuilderOptions {
}
/**
 * Special characters that can be used in matches.
 */
export declare enum Characters {
    ANY = 0,
    DIGIT = 1,
    NON_DIGIT = 2,
    ALPHANUM = 3,
    NON_ALPHANUM = 4,
    SPACE = 5,
    NON_SPACE = 6,
    HTAB = 7,
    VTAB = 8,
    RETURN = 9,
    LINEFEED = 10,
    FORMFEED = 11,
    BACKSPACE = 12,
    NULL = 13
}
export declare namespace Characters {
    /**
     * Match a control character.
     * @param caret {string} - is a one-length character from A-Z indicating the control character.
     */
    const CONTROL: (caret: string) => LiteralCharacterInterface;
    /**
     * Match a character by its character code
     * @param hex {string} - is a two-length or four-length string of hexadecimal digits indicating the character code.
     */
    const HEX: (hex: string) => LiteralCharacterInterface;
}
/**
 * Match a character literally, without escaping.
 * @param character {string} - the character being matched.
 */
export declare const LiteralCharacter: (character: string) => LiteralCharacterInterface;
export interface LiteralCharacterInterface {
    character: string;
}
/**
 * An assertion.
 */
export declare enum Assertion {
    START = 0,
    END = 1,
    WORD_BOUNDARY = 2,
    NOT_WORD_BOUNDARY = 3
}
/**
 * Options for a repeat.
 */
export declare namespace RepeatOptions {
    /**
     * Repeat zero or more times.
     * @param greedy {boolean} - if set to true, will match as many times as possible, otherwise as least times as possible.
     */
    const ZeroOrMore: (greedy?: boolean) => RepeatOptionsInterface;
    /**
     * Repeat one or more times.
     * @param greedy {boolean} - if set to true, will match as many times as possible, otherwise as least times as possible.
     */
    const OneOrMore: (greedy?: boolean) => RepeatOptionsInterface;
    /**
     * Repeat zero or one times.
     */
    const ZeroOrOne: () => RepeatOptionsInterface;
    /**
     * Repeat exactly some amount of times.
     * @param count {number} - the amount of times to repeat.
     */
    const Exactly: (count: number) => RepeatOptionsInterface;
    /**
     * Repeat at least some amount of times.
     * @param count {number} - the minimum amount of times to repeat.
     */
    const AtLeast: (count: number) => RepeatOptionsInterface;
    /**
     * Repeat between some amount of times and another amount of times (inclusive).
     * @param count1 {number} - the lower bounds for the amount of times to repeat.
     * @param count2 {number} - the upper bounds for the amount of times to repeat.
     */
    const Between: (count1: number, count2: number) => RepeatOptionsInterface;
}
export interface RepeatOptionsInterface {
    repeatChars: string;
}
export declare type Match = (character: Characters | LiteralCharacterInterface | string) => void;
export declare type Assert = (assertion: Assertion) => void;
export declare type EmptyCallback = (callback: Function) => void;
export declare type OR = (...callbacks: Function[]) => void;
export declare type BackRef = (index: number) => void;
export declare type Repeat = (callback: Function, options: RepeatOptionsInterface) => void;
