import { CustomTokenizer, RecursiveMap, RegexTokenizer, TokenizerChain } from "parselib";
import { Assertion, Characters, ExpressionBuilder } from "../builder";
/**
 * Compiles REXS into a regular expression string.
 * @param input {string} - the REXS code as a string.
 */
export var Compile = function (input) {
    var before = "";
    var beforeNot = "";
    var after = "";
    var afterNot = "";
    var flags = { i: false, g: false, m: false, s: false, u: false, y: false };
    return "/" + before + beforeNot + RecursiveMap(tokenizerChain.run(FixSemicolons(input)), function (input) { return !input.isToken && input.value === "{"; }, function (input) { return !input.isToken && input.value === "}"; }, function (input) {
        switch (input.value) {
            case "repeat":
                return "repeat-" + ProcessRepeatParams(input.data.split(",").map(function (x) { return x.trim().toLowerCase(); }));
            case "group":
                return "group";
            case "or":
                return "or";
            case "orpart":
                return "orpart";
            case "set":
                return "set-" + (input.data.split(",").map(function (x) { return x.trim().toLowerCase(); }).includes("not") ? "not" : "");
            case "before":
                return "before-" + (input.data.split(",").map(function (x) { return x.trim().toLowerCase(); }).includes("not") ? "not" : "");
            case "after":
                return "after-" + (input.data.split(",").map(function (x) { return x.trim().toLowerCase(); }).includes("not") ? "not" : "");
            case "flag":
                if (!Object.keys(flags).includes(input.data))
                    throw new Error("A flag was given that does not exist: " + input.data);
                flags[input.data] = true;
                return "";
            default:
                return HandleFunction(input);
        }
    }, function (input, startToken, endToken) {
        var split = input.shift().split("-");
        var name = split.shift();
        var data = split.join("-");
        switch (name) {
            case "repeat":
                var join = input.join("");
                if (join.length === 1)
                    return join + data;
                return "(?:" + join + ")" + data;
            case "group":
                return "(" + input.join("") + ")";
            case "or":
                return "(?:" + input.join("|") + ")";
            case "orpart":
                return input.join("");
            case "set":
                return "[" + (data === "not" ? "^" : "") + input.join("") + "]";
            case "before":
                if (data === "not")
                    beforeNot = "(?<!" + input.join("") + ")";
                else
                    before = "(?<=" + input.join("") + ")";
                return "";
            case "after":
                if (data === "not")
                    afterNot = "(?!" + input.join("") + ")";
                else
                    after = "(?=" + input.join("") + ")";
                return "";
            default:
                return input.join("");
        }
    }).join("") + after + afterNot + "/" + Object.keys(flags).filter(function (key) { return flags[key]; }).join("");
};
var HandleFunction = function (token) {
    var out = ExpressionBuilder(function (functions) {
        var functionsFormatted = Object.keys(functions).map(function (x) { return x.toLowerCase(); });
        var value = token.value.trim();
        var args = token.data.trim();
        if (!functionsFormatted.includes(value))
            throw new Error("An invalid function was given: " + token.value);
        switch (value) {
            case "match":
                var funValue = null;
                if ((args.startsWith("\"") && args.endsWith("\"")) || (args.startsWith("'") && args.endsWith("'"))) {
                    funValue = args.substring(1, args.length - 1);
                }
                else {
                    var val_1 = args.toUpperCase().split(",")[0];
                    if (!(val_1 in Characters))
                        throw new Error("An invalid character was given: " + token.data);
                    funValue = Characters[val_1];
                }
                if (typeof (funValue) === "function")
                    funValue = funValue(args.split(",")[1].trim());
                functions.Match(funValue);
                break;
            case "assert":
                var funValue1 = null;
                var val = args.toUpperCase();
                if (!(val in Assertion))
                    throw new Error("An invalid assertion was given: " + token.data);
                funValue1 = Assertion[val];
                functions.Assert(funValue1);
                break;
            case "to":
                functions.To();
                break;
            case "backref":
                functions.BackRef(args.trim());
                break;
        }
    });
    return out.substring(1, out.length - 1);
};
var ProcessRepeatParams = function (args) {
    var body = "";
    var greedy = "";
    if (args.length < 1)
        throw new Error("The repeat function requires arguments.");
    var from = ParseRepeatInt(args[0]);
    if (args.length > 1 && args[1] !== "nongreedy") {
        var to = ParseRepeatInt(args[1]);
        if (from === Infinity) {
            throw new Error("The \"from\" argument in repeat must be less than Infinity.");
        }
        if (from >= to) {
            throw new Error("The \"from\" argument in repeat must be less than the \"to\" argument.");
        }
        if (from < 0 || to < 0) {
            throw new Error("The \"from\" and \"to\" arguments in repeat must be greator than or equal to zero.");
        }
        switch ("" + from + to) {
            case "0Infinity":
                body = "*";
                break;
            case "1Infinity":
                body = "+";
                break;
            case "01":
                body = "?";
                break;
            default:
                if (to === Infinity) {
                    body = "{" + from + ",}";
                }
                else {
                    body = "{" + from + "," + to + "}";
                }
                break;
        }
    }
    else {
        body = "{" + from + "}";
    }
    if (args.includes("nongreedy"))
        greedy = "?";
    return body + greedy;
};
var ParseRepeatInt = function (int) {
    var cleaned = int.trim().toLowerCase();
    var parsed = parseInt(cleaned);
    if (!isNaN(parsed))
        return parsed;
    switch (cleaned) {
        case "inf":
        case "infinity":
        case "forever":
        case "more":
        case "max":
            return Infinity;
        case "one":
            return 1;
        case "zero":
        case "none":
            return 0;
    }
    throw new Error("Unknown argument for repeat: " + int);
};
var FixSemicolons = function (input) {
    return input.split("\n").map(function (val) {
        var fixed = val.trim();
        if (!fixed)
            return null;
        if ([";", "{", "}"].includes(fixed.substring(fixed.length - 1)))
            return fixed;
        return fixed + ";";
    }).filter(function (val) { return val !== null; }).join("");
};
var tokenizerChain = new TokenizerChain(new RegexTokenizer(/([a-zA-Z]*?)\s*?\((.*?)\)\s*?[;{]/g)).token(new CustomTokenizer(function (input) {
    var out = [];
    var split = input.trim().split("(");
    var name = split.shift();
    var join = split.join("(");
    if (join.endsWith("{")) {
        join = join.substring(0, join.length - 1);
        out.push({ value: "{", isToken: false });
    }
    var params = join.substring(0, join.length - 2);
    out.push({ value: name, data: params, isToken: true });
    return out;
})).text(new CustomTokenizer(function (input) {
    var out = [];
    while (input.startsWith("}") || input.startsWith("{")) {
        out.push({ value: input.substring(input.length - 1), isToken: false });
        input = input.substring(0, input.length - 1);
    }
    return out;
}));
//# sourceMappingURL=compiler.js.map